function Fig2_1
%bayesian optimal policy converges to the optimal policy for the stochastic
%HJB equation as n increases

nexact_all = [50:200:1000,2000,3000,5000];
error_pi = zeros(length(nexact_all),1);
error_w = zeros(length(nexact_all),1);

for stoch = 0:1
for ii = 1:length(nexact_all)
    nexact = nexact_all(ii);
    if stoch %stoch = 1; %f(n) = \sqrt{n}
        fn = sqrt(nexact);
    else %stoch = 0; %f(n) = n
        fn = nexact;
    end
    
    mu2 = 1/3/sqrt(nexact);
    a = nexact; b = nexact-sqrt(nexact);
    
    if stoch
        mu2_hat = 1/3; %stochastic
        a_hat = 1; b_hat = 2; 
    else
        mu2_hat = 0; %deterministic
        a_hat = 0; b_hat = 2; 
    end
    

    w = zeros(nexact+1,2*(nexact+1)-1);
    z = 0;
    for i = nexact:-1:1
        q = 0:i-1; q = q';
        s = (-(i-1):i-1); 
        p = (a+s/2+q/2)./(a+b+q); 
        temp1 = p.*w(2:end,3:end) + (1-p).* w(2:end,1:end-2) + 2*p-1;
        temp2 = w(1:end-1,2:end-1) + mu2;
        w = zeros(size(p));
        idx = temp1 >= temp2; w(idx) = temp1(idx);
        idx = ~idx; w(idx) = temp2(idx);
        
        muhat = (a_hat + s/fn )./(b_hat+q/nexact);
        idx_cts =  muhat < mu2_hat;
        error_pi(ii) = error_pi(ii) + sum(sum(abs(idx_cts - idx)));
        
        v = zeros(size(p)); t = (i-1)/nexact;
        v(idx_cts) = mu2_hat*(1-t); v(~idx_cts) = muhat(~idx_cts)*(1-t);
        error_w(ii) = error_w(ii) + sum(sum(abs(w/fn - v)));
        z = z+size(w,1)*size(w,2);
    end
    if stoch == 1
        error_w(ii) = error_w(ii)/z/sqrt(nexact);
    else
         error_w(ii) = error_w(ii)/z;
    end
    error_pi(ii) = error_pi(ii)/z;
    
end
figure(1); subplot(1,2,1); loglog(nexact_all,error_pi,'-*');hold on; 
figure(1); subplot(1,2,2); loglog(nexact_all,error_w,'-*');hold on; 
end



subplot(1,2,1); set(gca,'FontSize',15);
legend('$f(n) = n$','$f(n) = \sqrt{n}$','Interpreter','latex','FontSize',15); 
xlabel('n','Interpreter','latex','FontSize',20); 
title('$e^{n}_\pi$','Interpreter','latex','FontSize', 20)

subplot(1,2,2); set(gca,'FontSize',15);
legend('$f(n) = n$','$f(n) = \sqrt{n}$','Interpreter','latex','FontSize',15); 
xlabel('n','Interpreter','latex','FontSize',20); 
title('$e^{n}_w$','Interpreter','latex','FontSize', 20)


name = 'fig2_1';
print(gcf, '-depsc', name);
1;










