function Fig3_1
% compare three different algorithms
K = 5; fig_idx = 1; 
% K = 10; fig_idx = 2;
% K = 20; fig_idx = 3; 

nexact = 10^4;  %horizon
N_simu = 10^2; %number of simulations to get the expectation of the regret
sig = 1; %known of variance
muk = zeros(K,1); %unknown of expectation
muk(1) = 0; N_mu = 10; MU = linspace(-1,1,N_mu);
fn = sqrt(nexact); %scaling factor
a = zeros(K,1); b = zeros(K,1); ahat = zeros(K,1); bhat = zeros(K,1); %prior hyperparameters
a(1:end) = 1/sqrt(nexact); b(1:end) = 1/sqrt(nexact);%prior
ahat(1:end) = 1; bhat(1:end) = 1; 

if(1)%TS sampling
    n = nexact; T = n;
    R_exp = zeros(N_mu,1);
    for j = 1:N_mu
        muk(2:K) = MU(j); 
        mu = muk;
        R = zeros(N_simu,1);
        for simu = 1:N_simu
            s = zeros(K,1); q=zeros(K,1);
            for t = 1:T
                if t <= K && t >= 1 
                    R(simu) = R(simu) + mu(t);
                    r = normrnd(mu(t),sig);
                    s(t) = s(t)+r; q(t) = q(t)+1;
                else
                    %simulate mut based on the posterior
                    at = (a.*b.^(-2) + s)./(q + b.^(-2)); bt = 1./sqrt(q + b.^(-2));
                    mut = normrnd(at,bt);
                    %select the one has the max mut
                    [~,k] = max(mut);
                    R(simu) = R(simu)+mu(k);
                    %get reward at t, and update posterior pt
                    r = normrnd(mu(k),sig);
                    s(k) = s(k)+r; q(k) = q(k)+1;
                end
            end
        end   
        if mu(1) > mu(2)
            R = mu(1)*T - R;
        else
            R = mu(2)*T - R;
        end
        R_exp(j) = mean(R);
    end
    figure(2); h2 = subplot(1,3,fig_idx); cla(h2); y = plot(MU, R_exp,'LineWidth',2); legend(y, 'Thomson Sampling');hold on; 
    RTS_exp = R_exp;
end


if(1) %UCB
    n = nexact; T = n;
    R_exp = zeros(N_mu,1);
    for j = 1:N_mu
        muk(2:K) = MU(j); 
        mu = muk;
        R = zeros(N_simu,1);
        for simu = 1:N_simu
            s = zeros(K,1); q=zeros(K,1);
            for t = 1:T
                %decide pull which arm at time t
                if t <= K && t >= 1 
                    R(simu) = R(simu) + mu(t);
                    r = normrnd(mu(t),sig);
                    s(t) = s(t)+r; q(t) = q(t)+1;  
                else
                    muhat = s./q + sqrt(2./q.*log(n^2));
%                     muhat =  s./q + sqrt(3./q.*log(n/t));
                    %sqrt(2/q1*log(t)); sqrt(3/q*log(n/t)); sqrt(4/q1*log(max(1,n/(2*q1))))
                    [~,k] = max(muhat);
                    R(simu) = R(simu)+mu(k);
                    %get reward at t, and update posterior pt
                    r = normrnd(mu(k),sig);
                    s(k) = s(k)+r; q(k) = q(k)+1;
                end
            end
        end
        if mu(1) > mu(2)
            R = mu(1)*T - R;
        else
            R = mu(2)*T - R;
        end
        R_exp(j) = mean(R);
    end
    figure(2);hold on; subplot(1,3,fig_idx); y = plot(MU, R_exp,'LineWidth',2); legend(y, 'UCB');
    Rucb_exp = R_exp;
end


if(1) %approximated algo
    n = nexact; T = n;
    R_exp = zeros(N_mu,1);
    for j = 1:N_mu
        muk(2:K) = MU(j); 
        mu = muk;
        R = zeros(N_simu,1);
        for simu = 1:N_simu
            s = zeros(K,1); q=zeros(K,1);
            for t = 1:T
                if t <= K && t >= 1 
                    R(simu) = R(simu) + mu(t);
                    r = normrnd(mu(t),sig);
                    s(t) = s(t)+r; q(t) = q(t)+1;
                else
                    %decide pull which arm at time t
                    muhat = (s/fn+ahat)./(q/n+bhat);
                    [~,k] = max(muhat);
                    R(simu) = R(simu)+mu(k);
                    %get reward at t, and update posterior pt
                    r = normrnd(mu(k),sig);
                    s(k) = s(k)+r; q(k) = q(k)+1;
                end
            end
        end
        if mu(1) > mu(2)
            R = mu(1)*T - R;
        else
            R = mu(2)*T - R;
        end
        R_exp(j) = mean(R);
    end
    figure(2);hold on; subplot(1,3,fig_idx); y = plot(MU, R_exp,'LineWidth',2); legend(y, 'approximated HJB');
    Rhjb_exp = R_exp;
end

figure(2);subplot(1,3,fig_idx); xlabel('$\Delta$','Interpreter','latex'); 


subplot(1,3,1); title('K=5'); set(gca,'FontSize',20)
subplot(1,3,2); title('K=10'); set(gca,'FontSize',20)
subplot(1,3,3); title('K=20'); set(gca,'FontSize',20)
figure(2); sgtitle('The expected regret of the approximated bayesian algorithm, Thomson Sampling, UCB','FontSize', 25)
name = 'fig3_1';

print(gcf, '-depsc', name);

1;